; ##############################################################################
; ##############################################################################
; ##############################################################################
EXTERNER_INTERUPT_INITIALISIEREN:
;																			
; SLEEP+INT0+INT1 (externer Interrupt) konfigurieren						
;																			
; MCUCR => SE SM2 SM1 SM0 ISC11 ISC10 ISC01 ISC00 							
; SM2 SM1 SM0 Sleep Mode													
;*0   0   0   Idle															
; 0   0   1   ADC Noise Reduction											
; 0   1   0   Power-down													
; 0   1   1   Power-save													
; 1   0   0   Reserved														
; 1   0   1   Reserved														
; 1   1   0   Standby(1)													
;																			
;ISC11 ISC10 Description													
; 0 	0 	The low level of INT1 generates an interrupt request.			
; 0 	1 	Any logical change on INT1 generates an interrupt request.		
; 1 	0 	The falling edge of INT1 generates an interrupt request.		
; 1 	1 	The rising edge of INT1 generates an interrupt request.			
;																			
;ISC01 ISC00 Description													
; 0 	0 	The low level of INT0 generates an interrupt request.			
;*0 	1 	Any logical change on INT0 generates an interrupt request.		
; 1 	0 	The falling edge of INT0 generates an interrupt request.		
; 1 	1 	The rising  edge of INT0 generates an interrupt request.		
;																			
	ldi temp,(1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0)|(0<<ISC11)|(0<<ISC10)|(0<<ISC01)|(1<<ISC00) 
	out MCUCR,temp

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
EXTERNER_INTERRUPT_AKTIVIEREN:
;																			
; INT0+INT1 (externer Interrupt)aktivieren									
;																			
; GICR => INT1 INT0 - - - - IVSEL IVCE 										
; General Interrupt Control													
;																			
	ldi temp, (0<<INT1)|(1<<INT0)|(0<<IVSEL)|(0<<IVCE)   
    out GICR, temp 
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Dieser Interrupt wird bei jedem Flankenwechsel des 57kHz Signals aufgerufen.	
; Der RDS-DATA-PIN und RDS-TAKT-PIN geschaltet.									
; Der RDS
; 57.000Hz:48=1.187,5 Hz														
RDS_GENERATOR:
	out TCNT0,TIMER0WERT; berwachung / TIMER0 Startwert		
	in	 KOPIE_SREG,SREG; SICHERN SREG 	
	mov  KOPIE_TEMP,temp; SICHERN TEMP	
	mov  KOPIE_ZL, ZL	; SICHERN ZL	
	mov  KOPIE_ZH, ZH	; SICHERN ZH	
;===============================================================================
; ANALOG-AUSGANG (Signal invertieren / nicht invertieren)						
  	tst RDS_INVERTIERUNG                                   ;0
  	breq RDS_GENERATOR_NO_INVERTIERUNG                     ;1
  	sbis (AUSGANG_57kHz_PIN),(AUSGANG_57kHz_PIN_NR)        ;2
  	rjmp RDS_GENERATOR_PIN_HIGH                            ;3
  	rjmp RDS_GENERATOR_PIN_LOW                             ;4
RDS_GENERATOR_NO_INVERTIERUNG:
  	sbis (AUSGANG_57kHz_PIN),(AUSGANG_57kHz_PIN_NR)        ;3
  	rjmp RDS_GENERATOR_PIN_LOW                             ;4
RDS_GENERATOR_PIN_HIGH:        ;5 5
  	nop                                          ;6 6
  	sbi (AUSGANG_RDS_ANALOG_PORT),(AUSGANG_RDS_ANALOG_PIN_NR); Analog 57kHz        ;7 7
  	rjmp RDS_GENERATOR_PIN_FERTIG
RDS_GENERATOR_PIN_LOW:        ;6 6
  	cbi (AUSGANG_RDS_ANALOG_PORT),(AUSGANG_RDS_ANALOG_PIN_NR); Analog 57kHz        ;7 7
RDS_GENERATOR_PIN_FERTIG:
;===============================================================================
; 19kHz-Signal generieren														
	dec RDS_19kHz
	tst RDS_19kHz
	brne RDS_GENERATOR_19kHz_FERTIG
	ldi temp,3
	mov RDS_19kHz,temp
  	sbis (AUSGANG_19kHz_PIN),(AUSGANG_19kHz_PIN_NR)    
	rjmp RDS_GENERATOR_19kHz_HIGH
	rjmp RDS_GENERATOR_19kHz_LOW
RDS_GENERATOR_19kHz_HIGH:  
  	sbi (AUSGANG_19kHz_PORT),(AUSGANG_19kHz_PIN_NR);
	rjmp RDS_GENERATOR_19kHz_FERTIG
RDS_GENERATOR_19kHz_LOW:
  	cbi (AUSGANG_19kHz_PORT),(AUSGANG_19kHz_PIN_NR);
RDS_GENERATOR_19kHz_FERTIG:
;===============================================================================
; TAKTKORREKTUR, zur Generierung des 57kHz Signals, jittert aber etwas !						
; Kann entfallen, wenn externer Takt ein exaktes Vielfaches von 57kHz ist, z.B. 15.960.000 Hz	
;rjmp RDS_GENERATOR_TAKTKORREKTUR_FERTIG
	; Z-Initalisieren		
	ldi ZL,LOW (TABELLE_TAKTKORREKTUR_16MHz*2)
	ldi ZH,HIGH(TABELLE_TAKTKORREKTUR_16MHz*2)
	add ZL,RDS_ZAEHLER_TAKTKORREKTUR	; addieren			
	adc ZH,NULL
	LPM temp, Z   						; aus Tabelle laden 
	inc RDS_ZAEHLER_TAKTKORREKTUR		; +1				
	; Tabellenwert NULL ?	
	tst temp
	brne RDS_GENERATOR_TAKTKORREKTUR_w
	; Z-Initalisieren		
	ldi ZL,LOW (TABELLE_TAKTKORREKTUR_16MHz*2)
	ldi ZH,HIGH(TABELLE_TAKTKORREKTUR_16MHz*2)
	LPM temp, Z   				; aus Tabelle laden 
	clr RDS_ZAEHLER_TAKTKORREKTUR
	;-----------------------
RDS_GENERATOR_TAKTKORREKTUR_w:
	dec temp					; -1 (CTC)			
	out OCR2,temp              	; CTC				
RDS_GENERATOR_TAKTKORREKTUR_FERTIG:
;===============================================================================
; (Daten aus dem SRAM laden)													
	tst RDS_SINUS
	brne RDS_GENERATOR_DATA_FERTIG
	clr RDS_DATA
	; Z-Initalisieren	
	ldi ZL,LOW (adr_RDS_DATA_BUFFER13)
	ldi ZH,HIGH(adr_RDS_DATA_BUFFER13)
	; Aufbereitung Z	
	mov temp,RDS_ZAEHLER_BITS
	LSR temp	; 3x rechts			
	LSR temp
	LSR temp
	add ZL,temp	; addieren			
	adc ZH,NULL
	LD temp,Z	; aus SRAM laden	
	LSL temp	; 1 x links (Carry rollt raus)	
	ST Z,temp	; im SRAM ablegen	
	ROR RDS_DATA; 1 x rechts (Carry rollt rein)	
	tst RDS_DATA
	brne RDS_GENERATOR_DATA_HIGH
RDS_GENERATOR_DATA_LOW:
	cbi (AUSGANG_RDS_DATA_PORT)  ,(AUSGANG_RDS_DATA_PIN_NR)	; RDS-DATA		
	rjmp RDS_GENERATOR_DATA_FERTIG
RDS_GENERATOR_DATA_HIGH:
	sbi (AUSGANG_RDS_DATA_PORT)  ,(AUSGANG_RDS_DATA_PIN_NR)	; RDS-DATA		
RDS_GENERATOR_DATA_FERTIG:
;===============================================================================
; AUSGANG-RDS_TAKT																
	mov temp,RDS_SINUS
	cpi temp,(1)
	brne RDS_GENERATOR_TAKT1_FERTIG
	sbi (AUSGANG_RDS_TAKT_PORT)  ,(AUSGANG_RDS_TAKT_PIN_NR)	; RDS-TAKT (HIGH)	
RDS_GENERATOR_TAKT1_FERTIG:
;===============================================================================
; AUSGANG-RDS_TAKT																
	mov temp,RDS_SINUS
	cpi temp,(48)
	brne RDS_GENERATOR_TAKT2_FERTIG
	cbi (AUSGANG_RDS_TAKT_PORT)  ,(AUSGANG_RDS_TAKT_PIN_NR)	; RDS-TAKT (LOW)	
RDS_GENERATOR_TAKT2_FERTIG:
;===============================================================================
; ANALOG-AUSGANG																
; Wenn DATA=NULL dann wird SIGNAL Invertiert									
	mov temp,RDS_SINUS
	cpi temp,(47)
	brne RDS_GENERATOR_ANALOG_DATA_FERTIG
	tst RDS_DATA
	brne RDS_GENERATOR_ANALOG_DATA_FERTIG
	com RDS_INVERTIERUNG
RDS_GENERATOR_ANALOG_DATA_FERTIG:
;===============================================================================
	; ein Biphasesymbol besteht aus 48 Sinus-Schwingungen, jeder Flankenwechsel wird gezhlt (2x48=96)0...95		
	inc RDS_SINUS
	mov temp,RDS_SINUS
	cpi temp,(95+1)
	brlo RDS_GENERATOR_ZAEHLER_OK	
	clr RDS_SINUS
	; INVERTIERUNG				
	com RDS_INVERTIERUNG
	; RDS_ZAEHLER_BITS (Datenbits Gruppe A+B+C+D=4x(16+10)=104)  0...103	
	inc RDS_ZAEHLER_BITS
	mov temp,RDS_ZAEHLER_BITS
	cpi temp,(103+1)
	brlo RDS_GENERATOR_ZAEHLER_OK
	clr RDS_ZAEHLER_BITS
RDS_GENERATOR_ZAEHLER_OK:
;===============================================================================
	MOV ZH,KOPIE_ZH		; Wiederherstellen von ZH				
	MOV ZL,KOPIE_ZL		; Wiederherstellen von ZL				
	mov temp,KOPIE_TEMP	; Wiederherstellen von "TEMP" 			
	out	SREG,KOPIE_SREG	; Wiederherstellen von SREG				
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
