;												
; RDS-SIGNALGENERATOR fr ATmega8				
; Bernhard.Erfurt@gmx.de						
; 05/2014										
; VERSION 4										

;												
; Anschlussbelegung PINS						
;												
; PB0 ==> OUT RDS-Analog						
; PB1 ==> OUT TEST-Generator					
; PB2 ==> OUT 19kHz-GENERATOR					
; PB3 ==> OUT 57kHz-GENERATOR 					
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> Quarz									
; PB7 ==> Quarz									
;												
; PC0 ==> OUT RDS-TAKT							
; PC1 ==> OUT RDS-DATA							
; PC2 ==> 										
; PC3 ==> TASTE-C								
; PC4 ==> TASTE-B								
; PC5 ==> TASTE-A								
;												
; PD0 ==> USART (RX)							
; PD1 ==> 										
; PD2 ==> INPUT (vom 57kHz Generator)			
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==> LED GRN								
; PD6 ==> LED GELB								
; PD7 ==> LED ROT								
;												

;												
; LEDS:											
; GRN: Zeit fr Berechnungen					
; GELB: USART-MODUS								
; ROT: 	Zeitberschreitung im Interrupt			
; alle LEDs blinken: Interrupt-ERROR			
;												

;												
; USART-Protokoll (9Bytes)						
; "*", AH, AL,BH,BL,CH,CL,DH,DL 				
;												
; USART-Puffer clear							
; "*", 0,0,0,0,0,0,0,0,							
;												

; ATmega8 										
.include "m8def.inc"

; KONSTANTEN									
.equ TAKT							=16000000	
; USART											
.equ BAUD							=9600
; TIMER-WERTE										
.equ TIMERWERT0 					= 256-147	; 147 berwachungstimer ;(2 hoch 8  minus...)	
.equ TIMER1CTC						= 0			; 0=halber TAKT
.equ TIMER2CTC57kHz					= 139		; 57kHz (139 bei 15.960.000 Takt)	
; RDS-CRC-BERECHNUNG							
.equ  POLYNOM    					= 0x5B9
.equ  OFFSET_WORD_A  				= 0x0FC
.equ  OFFSET_WORD_B  				= 0x198
.equ  OFFSET_WORD_C  				= 0x168
.equ  OFFSET_WORD_D  				= 0x1B4
; EINGANG USART (RXD)							
.equ INP_RXD_PIN_NR					= 0
.equ INP_RXD_DDR					= DDRD
.equ INP_RXD_PORT					= PORTD
; TASTE-A									
.equ TASTE_A_PIN_NR					= 5
.equ TASTE_A_PIN					= PINC
.equ TASTE_A_DDR					= DDRC
.equ TASTE_A_PORT					= PORTC
; TASTE-B									
.equ TASTE_B_PIN_NR					= 4
.equ TASTE_B_PIN					= PINC
.equ TASTE_B_DDR					= DDRC
.equ TASTE_B_PORT					= PORTC
; TASTE-C									
.equ TASTE_C_PIN_NR					= 3
.equ TASTE_C_PIN					= PINC
.equ TASTE_C_DDR					= DDRC
.equ TASTE_C_PORT					= PORTC
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; AUSGANG-CTC (CTC-Timer1)						
.equ AUSGANG_CTC1_PIN_NR			= 1
.equ AUSGANG_CTC1_DDR				= DDRB
.equ AUSGANG_CTC1_PORT				= PORTB
; AUSGANG-57kHz (CTC-Timer2)					
.equ AUSGANG_57kHz_PIN_NR			= 3
.equ AUSGANG_57kHz_PIN				= PINB
.equ AUSGANG_57kHz_DDR				= DDRB
.equ AUSGANG_57kHz_PORT				= PORTB
; AUSGANG-19kHz									
.equ AUSGANG_19kHz_PIN_NR			= 2
.equ AUSGANG_19kHz_PIN				= PINB
.equ AUSGANG_19kHz_DDR				= DDRB
.equ AUSGANG_19kHz_PORT				= PORTB
; AUSGANG-RDS-57kHz-analog						
.equ AUSGANG_RDS_ANALOG_PIN_NR		= 0
.equ AUSGANG_RDS_ANALOG_DDR			= DDRB
.equ AUSGANG_RDS_ANALOG_PORT		= PORTB
; AUSGANG-RDS-TAKT								
.equ AUSGANG_RDS_TAKT_PIN_NR		= 0
.equ AUSGANG_RDS_TAKT_DDR			= DDRC
.equ AUSGANG_RDS_TAKT_PORT			= PORTC
; AUSGANG-RDS-DATA								
.equ AUSGANG_RDS_DATA_PIN_NR		= 1
.equ AUSGANG_RDS_DATA_DDR			= DDRC
.equ AUSGANG_RDS_DATA_PORT			= PORTC

; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						= SRAM_START		; 0x0060 START Adresse 		

.equ adr_RDS_MODUS					=SRAM_START+10		; 0=Tabelle 1=USART			

.equ adr_RDS_AH						=SRAM_START+50 		; GRUPPE 					
.equ adr_RDS_AL						=SRAM_START+51
.equ adr_RDS_BH						=SRAM_START+52
.equ adr_RDS_BL						=SRAM_START+53
.equ adr_RDS_CH						=SRAM_START+54
.equ adr_RDS_CL						=SRAM_START+55
.equ adr_RDS_DH						=SRAM_START+56
.equ adr_RDS_DL						=SRAM_START+57

.equ adr_RDS_CRC_A_H				=SRAM_START+80 		; CRC		
.equ adr_RDS_CRC_A_L				=SRAM_START+81
.equ adr_RDS_CRC_B_H				=SRAM_START+82 		
.equ adr_RDS_CRC_B_L				=SRAM_START+83 		
.equ adr_RDS_CRC_C_H				=SRAM_START+84 		
.equ adr_RDS_CRC_C_L				=SRAM_START+85 		
.equ adr_RDS_CRC_D_H				=SRAM_START+86 		
.equ adr_RDS_CRC_D_L				=SRAM_START+87

.equ adr_USART_HILFS_ZEIGER			=SRAM_START+100		; ZEIGER 					
.equ adr_USART_ZEIGER_WRITE			=SRAM_START+101		; ZEIGER 					
.equ adr_USART_ZEIGER_READ			=SRAM_START+102		; ZEIGER 					
.equ adr_USART_HILFSBUFFER8			=SRAM_START+110		; 8 BYTES DATENPUFFER		
.equ adr_USART_BUFFER256			=SRAM_START+120		; 256 BYTES DATENPUFFER		
 		
.equ adr_RDS_DATA_ZEIGER_L			= SRAM_START+600	; ZEIGER Datenpuffer		
.equ adr_RDS_DATA_ZEIGER_H			= SRAM_START+601

.equ adr_VORBUFFER13				= SRAM_START+800	; Vorpuffer					

.equ adr_RDS_DATA_BUFFER13			= SRAM_START+900	; Datenpuffer				

.equ adr_SRAM_CHECK					= SRAM_START+943	; SRAM CHECK muss immer "S" sein	

; REGISTER										
.def KOPIE_SREG			= R0	; SREG (INTERRUPT)
.def KOPIE_TEMP			= R1	; TEMP (INTERRUPT)
.def KOPIE_ZL			= R2
.def KOPIE_ZH			= R3
.def TIMER0WERT			= R4	; Kontrolltimer		
.def RDS_19kHz			= R5

.def RDS_SINUS			= R8
.def RDS_ZAEHLER_BITS	= R9
.def RDS_INVERTIERUNG	= R10
.def RDS_DATA			= R11
.def RDS_ZAEHLER_TAKTKORREKTUR	= R12	; CTC 				
.def NULL 				= R14
.def EINS 				= R15

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25


.cseg					;Beginn eines Code-Segmentes							
.org 0					;Startadresse=0											

rjmp RESET      		; Reset Handler                           				
rjmp RDS_GENERATOR 		; IRQ0 Handler                      					
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp TIM0_OVF   		; Timer0 Overflow Handler           					
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp USART_RXD 			; USART RX Complete Handler     						
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an	
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB an	
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	rcall wait_100ms
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus  
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	rcall wait_100ms
	rjmp INTERRUPT_ERROR

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
	rcall SRAM_CLEAR
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; TIMER0 Startwert	
	ldi temp,(TIMERWERT0)
	mov TIMER0WERT,temp
	; RDS				
	clr RDS_SINUS
	clr RDS_ZAEHLER_BITS
	clr RDS_ZAEHLER_TAKTKORREKTUR
	clr RDS_INVERTIERUNG
	; RDS DATA-Tabelle	
	ldi ZL,LOW (DATA*2)
	ldi ZH,HIGH(DATA*2)
	STS(adr_RDS_DATA_ZEIGER_L),ZL
	STS(adr_RDS_DATA_ZEIGER_H),ZH
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
; PORT B+C+C initialisieren							
	out DDRB, NULL
	out DDRC, NULL
	out DDRD, NULL
	out PORTB,NULL
	out PORTC,NULL
	out PORTD,NULL
; PULL-UP ein										
	sbi (INP_RXD_PORT),(INP_RXD_PIN_NR)							; RXD		
	sbi (TASTE_A_PORT),(TASTE_A_PIN_NR)							; TASTE-A	
	sbi (TASTE_B_PORT),(TASTE_B_PIN_NR)							; TASTE-B	
	sbi (TASTE_C_PORT),(TASTE_C_PIN_NR)							; TASTE-C	
; PORT LED auf AUSGANG								
	sbi (LED_GRUEN_DDR),(LED_GRUEN_PIN_NR)						; GRN		
	sbi (LED_GELB_DDR),(LED_GELB_PIN_NR)						; GELB		
	sbi (LED_ROT_DDR),(LED_ROT_PIN_NR)							; ROT		
; LEDs an											
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)						; GRN an	
	cbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)						; GELB an	
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)						; ROT  an	
; AUSGANG-A											
	sbi (AUSGANG_CTC1_DDR),(AUSGANG_CTC1_PIN_NR)				; TEST-Generator	
	sbi (AUSGANG_19kHz_DDR),(AUSGANG_19kHz_PIN_NR)				; 19kHz-Generator	
	sbi (AUSGANG_57kHz_DDR),(AUSGANG_57kHz_PIN_NR)				; 57kHz-Generator	

	sbi (AUSGANG_RDS_ANALOG_DDR),(AUSGANG_RDS_ANALOG_PIN_NR)	; RDS-Analog		
	sbi (AUSGANG_RDS_TAKT_DDR),(AUSGANG_RDS_TAKT_PIN_NR)		; RDS-TAKT			
	sbi (AUSGANG_RDS_DATA_DDR),(AUSGANG_RDS_DATA_PIN_NR)		; RDS-DATA			
; AUSGNGE auf LOW									
	cbi (AUSGANG_CTC1_PORT)  ,(AUSGANG_CTC1_PIN_NR)				; TEST-Generator	
	cbi (AUSGANG_19kHz_PORT) ,(AUSGANG_19kHz_PIN_NR)			; 19kHz-Generator	
	cbi (AUSGANG_57kHz_PORT) ,(AUSGANG_57kHz_PIN_NR)			; 57kHz-Generator	

	cbi (AUSGANG_RDS_ANALOG_PORT),(AUSGANG_RDS_ANALOG_PIN_NR)	; Analog 57kHz		
	cbi (AUSGANG_RDS_TAKT_PORT)  ,(AUSGANG_RDS_TAKT_PIN_NR)		; RDS-TAKT			
	cbi (AUSGANG_RDS_DATA_PORT)  ,(AUSGANG_RDS_DATA_PIN_NR)		; RDS-DATA			
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; LED-TEST											
	rcall LED_TEST
; 	PROGRAMM_CHECK									
	rcall PROGRAMM_CHECK
; EXTERNER-INTERUPT									
	rcall EXTERNER_INTERUPT_INITIALISIEREN
	rcall EXTERNER_INTERRUPT_AKTIVIEREN
; USART_INITIALISIEREN								
	rcall USART_INITIALISIEREN
; CTC-TIMER1+2										
	rcall TIMER1_CTC_INITIALISIERUNG
	rcall TIMER2_CTC_INITIALISIERUNG
	rcall TIMER0_INITIALISIERUNG
	rcall TIMER_AKTIVIEREN
	rcall TIMER_SYNCHRONISATION
; Interrupts freigeben								
	sei
; LEDS												
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR											; Watch-Dog-RESET		
	rcall RDS_DATEN
	rcall RDS_DATEN_QUELLE
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 13 Daten-Bytes 4x(16+10) vom VORBUFFER in den Hauptpuffer schreiben			
; Muss schnell gehen, nur wenige TAKTE Zeit !									
RDS_DATEN:
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	SLEEP										; pennen	
	tst RDS_ZAEHLER_BITS
	brne RDS_DATEN
	tst RDS_SINUS
	brne RDS_DATEN
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an	
;-------------------------------------------------------------------------------
	; X-Initalisieren	
	ldi XL,LOW (adr_VORBUFFER13)		; Vorpuffer		
	ldi XH,HIGH(adr_VORBUFFER13)
	; Y-Initalisieren	
	ldi YL,LOW (adr_RDS_DATA_BUFFER13)	; Hauptpuffer	
	ldi YH,HIGH(adr_RDS_DATA_BUFFER13)
	; SCHLEIFE			
	ldi temp9,13
RDS_DATEN_s:
	LD temp, X+   			; aus SRAM laden	
	ST Y+,temp				; im SRAM ablegen	
	dec temp9
	brne RDS_DATEN_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
RDS_DATEN_QUELLE:
;-------------------------------------------------------------------------------
	; TASTE-A gedrckt ?	
  	sbis (TASTE_A_PIN),(TASTE_A_PIN_NR)        
	rjmp RDS_DATEN_NULL
;-------------------------------------------------------------------------------
	; TASTE-B gedrckt ?	
  	sbis (TASTE_B_PIN),(TASTE_B_PIN_NR)        
	rjmp RDS_DATEN_EINS
;-------------------------------------------------------------------------------
	; TASTE-C gedrckt ?	
  	sbis (TASTE_C_PIN),(TASTE_C_PIN_NR)        
	rjmp RDS_DATEN_BITMUSTER
;-------------------------------------------------------------------------------
	; DATEN im USART-Puffer ?	
	LDS temp,(adr_USART_BUFFER256+0)
	tst temp
	breq RDS_DATEN_QUELLE_NO_USART_DATEN
	; ja, Daten im USART-Puffer	
	ldi temp,1
	STS(adr_RDS_MODUS),temp
RDS_DATEN_QUELLE_NO_USART_DATEN:
;-------------------------------------------------------------------------------
	LDS temp,(adr_RDS_MODUS)
	cpi temp,0
	breq RDS_DATEN_QUELLE_TABELLE
	cpi temp,1
	breq RDS_DATEN_QUELLE_USART
	ret
RDS_DATEN_QUELLE_TABELLE:
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR		; GELB aus				
	rjmp RDS_DATEN_TABELLE
RDS_DATEN_QUELLE_USART:
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR		; GELB an				
	rjmp RDS_DATEN_USART	
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Es werden nur NULLEN (Rohdaten) gesendet										
RDS_DATEN_NULL:
	; Z-initialisieren	
	ldi ZL,LOW (adr_VORBUFFER13)
	ldi ZH,HIGH(adr_VORBUFFER13)
	; Schleife			
	ldi temp9,13
RDS_DATEN_NULL_s:
	ldi temp,0	; NULL
	ST Z+,temp
	dec temp9
	brne RDS_DATEN_NULL_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Es werden nur EINSEN (Rohdaten) gesendet										
RDS_DATEN_EINS:
	; Z-initialisieren	
	ldi ZL,LOW (adr_VORBUFFER13)
	ldi ZH,HIGH(adr_VORBUFFER13)
	; Schleife			
	ldi temp9,13
RDS_DATEN_EINS_s:
	ldi temp,255	; alle BITS eins
	ST Z+,temp
	dec temp9
	brne RDS_DATEN_EINS_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Bitmuster (Rohdaten) senden													
RDS_DATEN_BITMUSTER:
	; Z-initialisieren	
	ldi ZL,LOW (adr_VORBUFFER13)
	ldi ZH,HIGH(adr_VORBUFFER13)
	; Schleife			
	ldi temp9,13
RDS_DATEN_BITMUSTER_s:
	ldi temp,0b01010101
	ST Z+,temp
	dec temp9
	brne RDS_DATEN_BITMUSTER_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
RDS_DATEN_USART:
	ldi temp,255
RDS_DATEN_USART_s:
	; Z-initialisieren				
	ldi ZL, LOW (adr_USART_BUFFER256)
	ldi ZH, HIGH(adr_USART_BUFFER256)
	LDS temp,(adr_USART_ZEIGER_READ)
	add ZL,temp
	adc ZH,NULL
	subi temp,-8	; +8
	STS(adr_USART_ZEIGER_READ),temp
	; Schleife beenden, wenn keine Daten	
	dec temp9
	breq RDS_DATEN_USART_RUN
	; sind Daten im Puffer ?		
	LD temp,Z
	tst temp
	breq RDS_DATEN_USART
RDS_DATEN_USART_RUN:
;-------------------------------------------------------------------------------
	LD temp,Z+					; AH aus SRAM laden		
	STS (adr_RDS_AH),temp		; im SRAM ablegen		
	LD temp,Z+					; AL aus SRAM laden		
	STS (adr_RDS_AL),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_BH),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_BL),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_CH),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_CL),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_DH),temp		; im SRAM ablegen		
	LD temp,Z+					; aus SRAM laden		
	STS (adr_RDS_DL),temp		; im SRAM ablegen		
	rcall RDS_ROHDATEN_BERECHNUNG
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Gruppe A+B+C+D aus Tabelle laden und CRC berechnen und Vorpuffer beschreiben	
RDS_DATEN_TABELLE:
	; Z-Initalisieren			
	LDS ZL,(adr_RDS_DATA_ZEIGER_L)
	LDS ZH,(adr_RDS_DATA_ZEIGER_H)
	LPM temp, Z   			; aus Tabelle laden 
	; Ende Tabelle erreicht ?	
	tst temp
	brne RDS_DATEN_TABELLE_w
	; Z-Initalisieren			
	ldi ZL,LOW (DATA*2)
	ldi ZH,HIGH(DATA*2)
RDS_DATEN_TABELLE_w:
;-------------------------------------------------------------------------------
	LPM temp1, Z+   			; aus Tabelle laden Z+1	(LOW)	
	LPM temp2, Z+   			;						(HIGH)	
	STS (adr_RDS_AL),temp1		; im SRAM ablegen		
	STS (adr_RDS_AH),temp2
	LPM temp1, Z+   			; aus Tabelle laden Z+1	
	LPM temp2, Z+   			
	STS (adr_RDS_BL),temp1		; im SRAM ablegen		
	STS (adr_RDS_BH),temp2
	LPM temp1, Z+   			; aus Tabelle laden Z+1	
	LPM temp2, Z+   			
	STS (adr_RDS_CL),temp1		; im SRAM ablegen		
	STS (adr_RDS_CH),temp2
	LPM temp1, Z+   			; aus Tabelle laden Z+1	
	LPM temp2, Z+   			
	STS (adr_RDS_DL),temp1		; im SRAM ablegen		
	STS (adr_RDS_DH),temp2
	rcall RDS_ROHDATEN_BERECHNUNG
;-------------------------------------------------------------------------------
	; Z-speichern				
	STS(adr_RDS_DATA_ZEIGER_L),ZL
	STS(adr_RDS_DATA_ZEIGER_H),ZH
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "RDS_SIGNAL_TIMER.asm"
.include "RDS_SIGNAL_EXTINTERRUPT.asm"
.include "RDS_USART.asm"
.include "RDS_SIGNAL_RDS.asm"
.include "RDS_SIGNAL_SONSTIGES.asm"			; sonstiges						
.include "RDS_SIGNAL_WAIT_16MHZ.asm"		; 16 MHz						
;-------------------------------------------------------------------------------
TABELLE_TAKTKORREKTUR_16MHz:	; 57 / 144kHz	
.db 140,141,140,140,141,140,140,141,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,140,141,140,141,140,140,141,140,140,141,140,0
;-------------------------------------------------------------------------------
DATA:
.dw 0xD4F6,0x4400,0x0000,0xE684	; ZEIT	
.dw 0xD4F6,0x0408,0x6564,0x4245
.dw 0xD4F6,0x0409,0x6564,0x524E
.dw 0xD4F6,0x040A,0x6564,0x4841
.dw 0xD4F6,0x040B,0x6564,0x5244
.dw 0xD4F6,0x2400,0x5244,0x532D
.dw 0xD4F6,0x2401,0x4765,0x6E65
.dw 0xD4F6,0x2402,0x7261,0x746F
.dw 0xD4F6,0x2403,0x7220,0x2020
.dw 0xD4F6,0x2404,0x3035,0x2F32
.dw 0xD4F6,0x2405,0x3031,0x3420
.dw 0xD4F6,0x2406,0x2020,0x4265
.dw 0xD4F6,0x2407,0x726E,0x6861
.dw 0xD4F6,0x2408,0x7264,0x2E45
.dw 0xD4F6,0x2409,0x7266,0x7572
.dw 0xD4F6,0x240A,0x7440,0x676D
.dw 0xD4F6,0x240B,0x782E,0x6465
.dw 0xD4F6,0x240C,0x2020,0x2020
.dw 0xD4F6,0x240D,0x2041,0x546D
.dw 0xD4F6,0x240E,0x6567,0x6138
.dw 0xD4F6,0x240F,0x2D31,0x364D
.dw 0xD4F6,0x2410,0x487A,0x2056
.dw 0xD4F6,0x2411,0x6572,0x7369
.dw 0xD4F6,0x2412,0x6F6E,0x3220
.dw 0xD4F6,0x2413,0x5461,0x6B74
.dw 0xD4F6,0x2414,0x3A20,0x3136
.dw 0xD4F6,0x2415,0x4D48,0x7A20
.dw 0xD4F6,0x2416,0x2030,0x3132
.dw 0xD4F6,0x2417,0x3334,0x3536
.dw 0xD4F6,0x2418,0x3738,0x3930
.dw 0xD4F6,0x2419,0x2041,0x4243
.dw 0xD4F6,0x241A,0x4445,0x4647
.dw 0xD4F6,0x241B,0x4849,0x4A4B
.dw 0xD4F6,0x241C,0x4C4D,0x4E4F
.dw 0xD4F6,0x241D,0x5051,0x5253
.dw 0xD4F6,0x241E,0x5455,0x5657
.dw 0xD4F6,0x241F,0x5859,0x5A21
.db 0x0000,0x0000,0x0000,0x0000 ; NULL







